#
# This script launches several runs of test_nsga in parallel.
# You must set:
# - SCRIPT_NAME executable
# - N_RUNS: number of runs for each configuration file
# - CONFIG_FILES: file containing parameters
# - OPTIMIZERS: [DE], [PSO] or [DE, PSO]
#

import multiprocessing
import subprocess, time
import itertools

def get_experiment_name(test_name, optimizer, pid):
    return test_name.split('/')[-1].split('.')[0] + '_' + optimizer + '_' + str(pid)

SCRIPT_NAME = './test_multi'
N_RUNS = 10
CONFIG_FILES = [
                '../config/seven_functions.conf',
               ]
OPTIMIZERS = ['PSO', 'DE']

ALL_CONFIGURATIONS = list()
for run in xrange(N_RUNS):
    for config in CONFIG_FILES:
        for optimizer in OPTIMIZERS:
            ALL_CONFIGURATIONS.append((config, optimizer, str(run)))

print 'Configurations that will be run'
for config in ALL_CONFIGURATIONS:
    print '\t', config

cores_num = multiprocessing.cpu_count()

pid = 0
running_processes = list()
num_tests = len(ALL_CONFIGURATIONS)
for _ in xrange(min(cores_num, num_tests)):
    test_name = ALL_CONFIGURATIONS[pid][0]
    optimizer = ALL_CONFIGURATIONS[pid][1]
    run_num   = ALL_CONFIGURATIONS[pid][2]
    print 'Starting test %d/%d' % (pid+1, num_tests), test_name, optimizer
    experiment_name = get_experiment_name(test_name, optimizer, run_num)
    running_processes.append( (test_name, subprocess.Popen([SCRIPT_NAME, optimizer, test_name, experiment_name])))
    pid += 1
while len(running_processes) > 0:
    for test_name, p in list(running_processes):
        if p.poll() is not None:
            running_processes.remove((test_name, p))
            print 'Test', test_name, 'completed!'
            if pid < num_tests:
                test_name = ALL_CONFIGURATIONS[pid][0]
                optimizer = ALL_CONFIGURATIONS[pid][1]
                run_num   = ALL_CONFIGURATIONS[pid][2]
                print 'Starting test %d/%d' % (pid+1, num_tests), test_name, optimizer
                experiment_name = get_experiment_name(test_name, optimizer, run_num)
                running_processes.append( (test_name, subprocess.Popen([SCRIPT_NAME, optimizer, test_name, experiment_name])))
                pid += 1
    time.sleep(20)
