import emopat_utils as utils

import argparse

parser = argparse.ArgumentParser(
    description='Extract/show results from a EMOPaT run'
)
parser.add_argument(
    'path',
    metavar='<path>',
    nargs='+', # uncomment to read path as a list
    type=str,
    help='path (or pattern, because more than one experiment can be selected) containing results of EMOPaT run'
)
parser.add_argument(
    '--header_file',
    metavar='<path>',
    #nargs='+', # uncomment to read path as a list
    default='../src/nsga2/NSGA2_DEPSO.h',
    type=str,
    help='C++ header file containing info about particles'
)
parser.add_argument(
    '--plot_2d',
    nargs='?',
    metavar='<bool>',
    type=bool,
    const=True,
    default=False,
    help='Plot the results'
)
parser.add_argument(
    '--plot_parameters',
    metavar='<int>',
    type=int,
    default=None,
    help='Plot the goal (passed here) on one axis and the parameters on the other'
)

parser.add_argument(
    '--print_pareto',
    nargs='?',
    metavar='<bool>',
    type=bool,
    const=True,
    default=False,
    help='Print the entire Pareto Front'
)
parser.add_argument(
    '--print_best_elements',
    metavar='<int>',
    type=int,
    default=0,
    help='Print N best elements'
)

args = parser.parse_args()

# Initialize experiment
experiment = utils.Experiment(args.path, args.header_file)

# Print results with rank 1
if args.print_pareto:
    print '*************************************************************'
    print '*************** PRINTING ENTIRE PARETO FRONT ****************'
    print '*************************************************************'
    experiment.print_population(rank_max=1)

# Print all best elements, according to each goal
if args.print_best_elements:
    print '*************************************************************'
    print '*************** PRINTING BEST ELEMENTS **********************'
    print '*************************************************************'
    experiment.print_best_elements(num=args.print_best_elements)

# Plot results of the first two goals
if args.plot_2d:
    print '*************************************************************'
    print '*************** PLOTTING TWO GOALS **************************'
    print '*************************************************************'
    experiment.plot_2D()

# Plot the goal on one axis and the parameters on the other
if args.plot_parameters is not None:
    print '*************************************************************'
    print '*************** PLOTTING TWO GOALS **************************'
    print '*************************************************************'
    experiment.plot_parameters(args.plot_parameters)
