#ifndef __COMMON_H__
#define __COMMON_H__

/* This file contains structures common to DE and PSO */

typedef enum
{
    STOP_NEVER = 0,  ///< Not set
	STOP_EVALS = 1,  ///< Use number of evaluations as termination criteria
	STOP_GENS  = 2,  ///< Use number of generations as termination criteria
	STOP_FIT   = 4,  ///< Use fitness value as termination criteria
	STOP_TIME  = 8,  ///< Use time as termination criteria (Not implemented)
    STOP_SS    = 16, ///< Use Steady State as termination criteria
} StoppingCriterion;

// good grief...
inline StoppingCriterion operator|(StoppingCriterion a, StoppingCriterion b)
{return static_cast<StoppingCriterion>(static_cast<int>(a) | static_cast<int>(b));}


/// Struct that contains results of the evolution
struct DEPSO_RESULTS {
	double best_fitness;
	int convergence_gen;
	int convergence_evals;
};

/*
/// A fitness function. (DEPRECATED ones)
/// @param[in] size the number of variables.
/// @param[in] parameters an array of size variables.
/// @param[in] data the custom data, untouched
///        as set in DEPSOFitnessFunction.
/// @returns the fitness.
// int size, double * parameters, void * data
#define DEPSO_FITNESS_FUNCTION(x) double (* x)(int,double *,void *)

/// This structure represent a fitness function, with
/// custom data associated.
struct DEPSOFitnessFunction {
    DEPSO_FITNESS_FUNCTION(func);
    void * data;
};
*/

/// Represents the boundaries for a fitness function.
/// Use PSOInitBoundaries to initialize and
/// PSODestroyBoundaries to destroy.
struct DEPSOBoundaries {
    /// The number of variables.
    int size;
    /// An array of size maximum values.
    double * max;
    /// An array of size minimum values.
    double * min;
};

/// Represents a position in the space of the problem,
/// i. e. an individual.
/// Use PSO(DE)InitPosition to initialize,
/// PSO(DE)DestroyPosition to destroy,
/// PSO(DE)CopyPosition to copy.
struct DEPSOPosition {
    /// The number of variables.
    int size;
    /// The current fitness of the individual.
    double fitness;
    /// The current position of the individual.
    double * x;
};

#endif
