#include "de_crossover_strategies.h"

#include "random.h"
#include "random_subgroup.h"

DE_CROSSOVER_STRATEGY(const de_crossover_strategy_functions[DE_META_MUTATION_COUNT][DE_META_CROSSOVER_COUNT]) = {
    {
        de_crossover_strategy_best_1_bin,
        de_crossover_strategy_best_1_exp
    },
    {
        de_crossover_strategy_targettobest_1_bin,
        de_crossover_strategy_targettobest_1_exp
    },
    {
        de_crossover_strategy_rand_1_bin,
        de_crossover_strategy_rand_1_exp
    }
};

void de_crossover_strategy_best_1_exp(int population_size, int problem_dimension,
    struct DEPSOPosition * best, struct DEPSOPosition * population,struct DEPSOPosition *individual,
    int id, void * data) {
    int n,L;
    int r2,r3;
    int exceptions[2];
    struct DECrossoverStrategyParameters * params = (struct DECrossoverStrategyParameters *) data;

    exceptions[0] = id;
    r2 = extract_random_index_except(population_size,1,exceptions);
    insert_index_ordered_asc(1,exceptions,r2);
    r3 = extract_random_index_except(population_size,2,exceptions);

    n = alea_integer(0,problem_dimension - 1);
    L = 0;
    do {
        individual->x[n] = best->x[n] + params->F * (population[r2].x[n] - population[r3].x[n]);
        n = (n + 1) % problem_dimension;
        L++;
    } while((alea_double(0,1) < params->CR) && (L < problem_dimension));
}

void de_crossover_strategy_rand_1_exp(int population_size, int problem_dimension,
    struct DEPSOPosition * best, struct DEPSOPosition * population,struct DEPSOPosition *individual,
    int id, void * data) {
    int n,L;
    int r1,r2,r3;
    int exceptions[3];
    struct DECrossoverStrategyParameters * params = (struct DECrossoverStrategyParameters *) data;

    exceptions[0] = id;
    r1 = extract_random_index_except(population_size,1,exceptions);
    insert_index_ordered_asc(1,exceptions,r1);
    r2 = extract_random_index_except(population_size,1,exceptions);
    insert_index_ordered_asc(2,exceptions,r2);
    r3 = extract_random_index_except(population_size,3,exceptions);

    n = alea_integer(0,problem_dimension - 1);
    L = 0;
    do {
        individual->x[n] = population[r1].x[n] + params->F*(population[r2].x[n] - population[r3].x[n]);
        n = (n+1)%problem_dimension;
        L++;
    } while((alea_double(0,1) < params->CR) && (L < problem_dimension));
}

void de_crossover_strategy_targettobest_1_exp(int population_size, int problem_dimension,
    struct DEPSOPosition * best, struct DEPSOPosition * population,struct DEPSOPosition *individual,
    int id, void * data) {
    int n,L;
    int r1,r2;
    int exceptions[2];
    struct DECrossoverStrategyParameters * params = (struct DECrossoverStrategyParameters *) data;

    exceptions[0] = id;
    r1 = extract_random_index_except(population_size,1,exceptions);
    insert_index_ordered_asc(1,exceptions,r1);
    r2 = extract_random_index_except(population_size,1,exceptions);

    n = alea_integer(0,problem_dimension - 1);
    L = 0;
    do {
        individual->x[n] = individual->x[n] + params->F*(best->x[n] - individual->x[n]) +
            params->F*(population[r1].x[n] - population[r2].x[n]);
        n = (n+1)%problem_dimension;
        L++;
    } while((alea_double(0,1) < params->CR) && (L < problem_dimension));
}

void de_crossover_strategy_best_1_bin(int population_size, int problem_dimension,
    struct DEPSOPosition * best, struct DEPSOPosition * population,struct DEPSOPosition *individual,
    int id, void * data) {
    int n,L;
    int r1,r2;
    int exceptions[2];
    struct DECrossoverStrategyParameters * params = (struct DECrossoverStrategyParameters *) data;

    exceptions[0] = id;
    r1 = extract_random_index_except(population_size,1,exceptions);
    insert_index_ordered_asc(1,exceptions,r1);
    r2 = extract_random_index_except(population_size,1,exceptions);

    n = alea_integer(0,problem_dimension - 1);
    // perform D binomial trials
    for (L = 0; L < problem_dimension; L++) {
        // change at least one parameter
        if ((alea_double(0,1) < params->CR) || L == (problem_dimension - 1)) {
            individual->x[n] = best->x[n] + params->F*(population[r1].x[n] - population[r2].x[n]);
        }
        n = (n + 1) % problem_dimension;
     }
}

void de_crossover_strategy_rand_1_bin(int population_size, int problem_dimension,
    struct DEPSOPosition * best, struct DEPSOPosition * population,struct DEPSOPosition *individual,
    int id, void * data) {
    int n,L;
    int r1,r2,r3;
    int exceptions[3];
    struct DECrossoverStrategyParameters * params = (struct DECrossoverStrategyParameters *) data;

    exceptions[0] = id;
    r1 = extract_random_index_except(population_size,1,exceptions);
    insert_index_ordered_asc(1,exceptions,r1);
    r2 = extract_random_index_except(population_size,1,exceptions);
    insert_index_ordered_asc(2,exceptions,r2);
    r3 = extract_random_index_except(population_size,3,exceptions);

    n = alea_integer(0,problem_dimension - 1);
    // perform D binomial trials
    for (L = 0; L < problem_dimension; L++) {
        // change at least one parameter
        if ((alea_double(0,1) < params->CR) || L == (problem_dimension - 1)) {
            individual->x[n] = population[r1].x[n] + params->F*(population[r2].x[n] - population[r3].x[n]);
        }
        n = (n + 1) % problem_dimension;
     }
}

void de_crossover_strategy_targettobest_1_bin(int population_size, int problem_dimension,
    struct DEPSOPosition * best, struct DEPSOPosition * population,struct DEPSOPosition *individual,
    int id, void * data) {
    int n,L;
    int r1,r2;
    int exceptions[2];
    struct DECrossoverStrategyParameters * params = (struct DECrossoverStrategyParameters *) data;

    exceptions[0] = id;
    r1 = extract_random_index_except(population_size,1,exceptions);
    insert_index_ordered_asc(1,exceptions,r1);
    r2 = extract_random_index_except(population_size,1,exceptions);

    n = alea_integer(0,problem_dimension - 1);
    // perform D binomial trials
    for (L = 0; L < problem_dimension; L++) {
        // change at least one parameter
        if ((alea_double(0,1) < params->CR) || L == (problem_dimension - 1)) {
            individual->x[n] = individual->x[n] + params->F*(best->x[n] - individual->x[n]) +
                params->F*(population[r1].x[n] - population[r2].x[n]);
        }
        n = (n + 1) % problem_dimension;
     }
}
