/*
 * This file is deprecated. It only contains some functions that I have
 * to implement in the CEC format
 */

#include <math.h>
#include <stdlib.h>

#include "fitness_functions.h"

double sphere_fitness_function(int size,double * x,void * data) {
    double result = 0;
    double temp;
    int i;

    for (i = 0; i < size; i++) {
        temp = x[i];
        result += temp * temp;
    }

    return result;
}

double dejong_fitness_function(int size,double * x,void * data) {
    double f = 0;
    double p;
    double xd;
    int i;

    p = 0; // Shift
    for (i = 0; i < size; i++){
        xd = x[i] - p;
        f += (i+1)*xd*xd*xd*xd;
    }

    return f;
}

double griewank_fitness_function(int size,double * x,void * data) {
    double f = 0;
    double p;
    double xd;
    int d;

    p = 1;
    for (d = 0; d < size; d++){
        xd = x[d];
        f = f + xd * xd;
        p = p * cos( xd / sqrtf( d + 1 ) );
    }
    f = f / 4000 - p + 1;

    return f;
}

double shifted_griewank_fitness_function(int size,double * x,void * data) {
    double f = 0;
    double p;
    double xd;
    int d;

    p = 1;
    for (d = 0; d < size; d++){
        //xd = (x[d] - (double)data[d]);
        xd = (x[d] - 74.32);
        f = f + xd * xd;
        p = p * cos( xd / sqrtf( d + 1 ) );
    }
    f = f / 4000 - p + 1;

    return f;
}

double rosenbrock_fitness_function(int size,double * x,void * data) {
    double f = 0;
    double t0;
    double t1,tt;
    int d;

    t0 = x[0];
    for ( d = 1; d < size; d++ ){
        t1 = x[d];
        tt = 1.0 - t0;
        f += tt * tt;
        tt = t1 - t0 * t0;
        f += 100.0 * tt * tt;
        t0 = t1;
    }

    return f;
}

double step_fitness_function(int size,double * x,void * data) {
    double f = 0;
    int d;

    for (d = 0; d < size; d++)
        f = f + (int)x[d];

    return f;
}

double foxholes2D_fitness_function(int size,double * x,void * data) {
    double f = 0;
    double sum1;
    int j;
    int d;

    static int a[2] [25] = {
        {-32, -16, 0, 16, 32, -32, -16, 0, 16, 32, -32, -16, 0, 16, 32, -32, -16, 0, 16, 32, -32, -16, 0, 16, 32},
        {-32, -32, -32, -32, -32, -16, -16, -16, -16, -16, 16, 16, 16, 16, 16, 32, 32, 32, 32, 32}
    };

    for (j = 0; j < 25; j++){
        sum1 = 0;
        for (d = 0; d < 2; d++){
            sum1 = sum1 + pow(x[d] - a[d][j],6);
        }
        f = f + 1 / (j + 1 + sum1);
    }
    f = 1 / ( 0.002 + f );

    return f;
}

double polynomial9D_fitness_function(int size,double * x,void * data) {
    int const M = 60;
    double py, y = -1, dx = (double)M;
    double f = 0;
    int d;
    int i;

    dx = 2 / dx;
    for ( i = 0; i <= M; i++ ){
        py = x[0];
        for ( d = 1; d < size; d++ ){
            py = y * py + x[d];
        }
        if ( py < -1 || py > 1 )
            f += ( 1 - py ) * ( 1 - py );
        y += dx;
    }
    py = x[0];
    for ( d = 1; d < size; d++ )
        py = 1.2 * py + x[d];
    py = py - 72.661;
    if ( py < 0 )
        f += py * py;
    py = x[0];
    for ( d = 1; d < size; d++ )
        py = -1.2 * py + x[d];
    py = py - 72.661;
    if ( py < 0 )
        f += py * py;

    return f;
}

double alpine_fitness_function(int size,double * x,void * data) {
    double f = 0;
    double xd;
    int d;

    for (d = 0; d < size; d++){
        xd = x[d];
        f += fabs(xd * sin(xd) + 0.1 * xd);
    }

    return f;
}

double rastrigin_fitness_function(int size,double * x,void * data) {
    double f = 0;
    double xd;
    int d;

    const double k = 10.0;
    for ( d = 0; d < size; d++ ){
        xd = x[d];
        f += xd * xd - k * cos(2.0 * M_PI * xd);
    }
    f += (double)size * k;

    return f;
}

double ackley_fitness_function(int size,double * x,void * data) {
    double f = 0.0;
    double xd;
    int d;
    double sum1=0.0,sum2=0.0;
    double y;

    for (d = 0; d < size; d++){
        xd = x[d];
        sum1 += xd * xd;
        sum2 += cos( 2.0 * M_PI * xd );
    }
    y = size;
    f = -20.0 * exp(-0.2 * sqrt( sum1 / y )) - exp(sum2 / y) + 20.0 + M_E;

    return f;
}

double tripod2D_fitness_function(int size,double * x,void * data) {
    double f = 0;
    double x1,x2;
    int d;
    double sum1,sum2;
    double y;

    x1 = x[0];
    x2 = x[1];
    if ( x2 < 0 ){
        f = fabs( x1 ) + fabs( x2 + 50 );
    }
    else{
        if ( x1 < 0 )
            f = 1 + fabs( x1 + 50 ) + fabs( x2 - 50 );
        else
            f = 2 + fabs( x1 - 50 ) + fabs( x2 - 50 );
    }

    return f;
}

double krishnakumar_fitness_function(int size,double * x,void * data) {
    double f = 0;
    int d;

    for ( d = 0; d < size - 1; d++ ){
        f = f + sin( x[d] + x[d + 1] ) + sin( 2 * x[d] * x[d + 1] / 3 );
    }

    return f;
}

double eason2D_fitness_function(int size,double * x,void * data) {
    double f = 0;
    double x1,x2;

    x1 = x[0];
    x2 = x[1];
    f = -cos(x1) * cos(x2) / exp( ( x1 - M_PI) * ( x1 - M_PI) + ( x2 - M_PI) * ( x2 - M_PI) );

    return f;
}

double zakharov_fitness_function(int size,double * x,void * data) {
    double f = 0.0;
    double tf;
    int i;

    for (i = 0; i < size; i++)
        f += x[i] * x[i];

    tf = 0.0;
    for (i = 0; i < size; i++)
        tf += 0.5*(i + 1)*x[i]*x[i];
    f += tf * tf;

    f += tf * tf * tf * tf;

    return f;
}

double schwefel_1_2_fitness_function(int size,double * x,void * data) {
    double f = 0.0;
    double tf;
    int i,h;

    for (i = 0; i < size; i++) {
        tf = 0.0;
        for (h = 0; h <= i; h++)
            tf += x[h];
        f += tf*tf;
    }

    return f;
}

double salomon_fitness_function(int size,double * x,void * data) {
    double f = 0.0, sum = 0.0;
    int i;
    for (i = 0; i < size; i++) {
        sum += x[i] * x[i];
    }
    sum = sqrt(sum);
    f = -cos(2.0 * M_PI * sum) + 0.1 * sum + 1;
    return f;
}

double schwefel_2_22_fitness_function(int size,double * x,void * data) {
    double sum = 0.0, prod = 1.0, absx, f;
    int i;
    for (i = 0; i < size; i++) {
        absx = fabs(x[i]);
        sum += absx;
        prod *= absx;
    }
    f = sum + prod;
    return f;
}

double scaffer_fitness_function(int size,double * x,void * data) {
    double f=0.0, tmp0, tmp1, tmp2;
    int i;
    //first element
    tmp0 = x[0] * x[0] + x[size-1] * x[size-1];
    tmp1 = sqrtf(tmp0);
    tmp2 = sin(tmp1) * sin(tmp1);
    f = 0.5 + (tmp2 - 0.5) / (1.0 + 0.0001 * tmp0);
    for (i = 1; i < size; i++) {
        tmp0 = x[i] * x[i] + x[i-1] * x[i-1];
        tmp1 = sqrtf(tmp0);
        tmp2 = sin(tmp1) * sin(tmp1);
        f += 0.5 + (tmp2 - 0.5) / (1.0 + 0.0001 * tmp0);
    }
    return f;
}

double levy_fitness_function(int size,double * x,void * data) {
    double f, yi, tmp;
    tmp = sin(M_PI * (1.0 + (x[0] - 1.0) / 4.0));
    f = tmp * tmp;
    for (int i = 0; i < size - 1; i++) {
        yi = 1.0 + (x[i] - 1.0) / 4.0;
        tmp = sin(M_PI * yi + 1.0);
        f += (yi - 1.0) * (yi - 1.0) * (10.0 * tmp * tmp);
    }
    yi = 1.0 + (x[size - 1] - 1.0) / 4.0;
    tmp = sin(2.0 * M_PI * yi);
    f += (yi - 1.0) * (yi - 1.0) * (1.0 + tmp * tmp);
    return f;
}

double dixonprice_fitness_function(int size,double * x,void * data) {
    double f = (x[0] - 1.0) * (x[0] - 1.0);
    double tmp;
    for (int i = 1; i < size; i++) {
        tmp = 2.0 * x[i] * x[i] - x[i-1];
        f += i * tmp * tmp;
    }
    return f;
}

double hyperellipsoid_fitness_function(int size,double * x,void * data) {
    double f = 0.0;
    for (int i = 0; i < size; i++) {
        f += i * i * x[i] * x[i];
    }
    return f;
}
