#include "test_func.h"
#include <cstdlib>

#define INF 1.0e99
#define EPS 1.0e-14
#define E  2.7182818284590452353602874713526625
#define PI 3.1415926535897932384626433832795029
#define TWOPI 6.2831853071795862319959269370883703

double *OShift,*M,*y,*z,*x_bound;;
double *w_cf_func;
int ini_flag,n_flag,func_flag;

void boundaries_by_function_name(std::string name, double & bottom, double & top)
{
    bottom = -100.0; top = 100.0; return;
}

void boundaries_by_function_id(int id, double & bottom, double & top)
{
    bottom = -100.0; top = 100.0; return;
}

int get_function_id_by_name(std::string name)
{
    if(name == "griewank")               {return GRIEWANK_ID;               }
    if(name == "rotated_griewank")       {return ROTATED_GRIEWANK_ID;       }
    if(name == "sphere")                 {return SPHERE_ID;                 }
    if(name == "zakharov")               {return ZAKHAROV_ID;               }
    if(name == "schwefel_1_2")           {return SCHWEFEL_1_2_ID;           }
    if(name == "rosenbrock")             {return ROSENBROCK_ID;             }
    if(name == "rastrigin")              {return RASTRIGIN_ID;              }
    if(name == "rotated_rosenbrock")     {return ROTATED_ROSENBROCK_ID;     }
    if(name == "rotated_rastrigin")      {return ROTATED_RASTRIGIN_ID;      }
    if(name == "salomon")                {return SALOMON_ID;                }
    if(name == "ackley")                 {return ACKLEY_ID;                 }
    if(name == "rotated_ackley")         {return ROTATED_ACKLEY_ID;         }
    if(name == "schwefel_2_22")          {return SCHWEFEL_2_22_ID;          }
    if(name == "scaffer")                {return SCAFFER_ID;                }
    if(name == "levy")                   {return LEVY_ID;                   }
    if(name == "alpine")                 {return ALPINE_ID;                 }
    if(name == "dixonprice")             {return DIXONPRICE_ID;             }
    if(name == "hyperellipsoid")         {return HYPERELLIPSOID_ID;         }
    if(name == "elliptic")               {return ELLIPTIC_ID;               }
    if(name == "rotated_elliptic")       {return ROTATED_ELLIPTIC_ID;       }
    if(name == "rotated_cigar")          {return ROTATED_CIGAR_ID;          }
    if(name == "rotated_discus")         {return ROTATED_DISCUS_ID;         }
    if(name == "powers")                 {return POWERS_ID;                 }
    if(name == "rotated_schaffers")      {return ROTATED_SCHAFFERS_F7_ID;   }
    if(name == "rotated_weierstrass")    {return ROTATED_WEIERSTRASS_ID;    }
    if(name == "rotated_step_rastrigin") {return ROTATED_STEP_RASTRIGIN_ID; }
    if(name == "schwefel")               {return SCHWEFEL_ID;               }
    if(name == "rotated_schwefel")       {return ROTATED_SCHWEFEL_ID;       }
    if(name == "rotated_katsuura")       {return ROTATED_KATSUURA_ID;       }
    if(name == "lunacek")                {return LUNACEK_ID;                }
    if(name == "rotated_lunacek")        {return ROTATED_LUNACEK_ID;        }
    if(name == "grie_rosen")             {return GRIE_ROSEN_ID;             }
    if(name == "cec_comp1")              {return COMP1_ID;                  }
	if(name == "cec_comp2")              {return COMP2_ID;                  }
	if(name == "cec_comp3")              {return COMP3_ID;                  }
	if(name == "cec_comp4")              {return COMP4_ID;                  }
	if(name == "cec_comp5")              {return COMP5_ID;                  }
	if(name == "cec_comp6")              {return COMP6_ID;                  }
	if(name == "cec_comp7")              {return COMP7_ID;                  }
	if(name == "cec_comp8")              {return COMP8_ID;                  }
	std::cout << "Unexisting function " << name << "!" << std::endl;
	exit(-1);
}


void sphere_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Sphere */
{
	shiftfunc(x, y, nx, Os);
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (int i=0; i<nx; i++)
		z[i]=y[i];
	f[0] = 0.0;
    for (int i=0; i<nx; i++)
    {
        f[0] += z[i]*z[i];
    }
}

void ellips_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Ellipsoidal */
{
	shiftfunc(x, y, nx, Os);
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (int i=0; i<nx; i++)
		z[i]=y[i];
    oszfunc (z, y, nx);
	f[0] = 0.0;
    for (int i=0; i<nx; i++)
    {
        f[0] += pow(10.0,6.0*i/(nx-1))*y[i]*y[i];
    }
}

void bent_cigar_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Bent_Cigar */
{
    int i;
	double beta=0.5;
	shiftfunc(x, y, nx, Os);
	if (r_flag==1)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];
    asyfunc (z, y, nx,beta);
	if (r_flag==1)
	rotatefunc(y, z, nx, &Mr[nx*nx]);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

	f[0] = z[0]*z[0];
    for (i=1; i<nx; i++)
    {
        f[0] += pow(10.0,6.0)*z[i]*z[i];
    }
}

void discus_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Discus */
{
    int i;
	shiftfunc(x, y, nx, Os);
	if (r_flag==1)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];
    oszfunc (z, y, nx);

	f[0] = pow(10.0,6.0)*y[0]*y[0];
    for (i=1; i<nx; i++)
    {
        f[0] += y[i]*y[i];
    }
}

void dif_powers_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Different Powers */
{
	shiftfunc(x, y, nx, Os);
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (int i=0; i<nx; i++)
		z[i]=y[i];
	f[0] = 0.0;
    for (int i=0; i<nx; i++)
    {
        f[0] += pow(fabs(z[i]),2+4*i/(nx-1));
    }
	f[0]=pow(f[0],0.5);
}

#define ROSENBROCK_SHRINK 0.02048
void rosenbrock_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Rosenbrock's */
{
	double tmp1,tmp2;
	shiftfunc(x, y, nx, Os);//shift
	for (int i=0; i<nx; i++)//shrink to the orginal search range
    {
        y[i] *= ROSENBROCK_SHRINK;
    }
	if (r_flag)
	rotatefunc(y, z, nx, Mr);//rotate
	else
    for (int i=0; i<nx; i++)
		z[i]=y[i];
	for (int i=0; i<nx; i++)//shift to orgin
    {
        z[i]=z[i]+1;
    }

    f[0] = 0.0;
    for (int i=0; i<nx-1; i++)
    {
		tmp1=z[i]*z[i]-z[i+1];
		tmp2=z[i]-1.0;
        f[0] += 100.0*tmp1*tmp1 +tmp2*tmp2;
    }
}

void schaffer_F7_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Schwefel's 1.2  */
{
    int i;
	shiftfunc(x, y, nx, Os);
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];
	asyfunc (z, y, nx, 0.5);
	for (i=0; i<nx; i++)
		z[i] = y[i]*pow(10.0,1.0*i/(nx-1)/2.0);
	if (r_flag)
	rotatefunc(z, y, nx, &Mr[nx*nx]);
	else
    for (i=0; i<nx; i++)
		y[i]=z[i];

	for (i=0; i<nx-1; i++)
		z[i]=pow(y[i]*y[i]+y[i+1]*y[i+1],0.5);
    f[0] = 0.0;
    for (i=0; i<nx-1; i++)
    {
	  double tmp=sin(50.0*pow(z[i],0.2));
	  double powz = pow(z[i],0.5);
	  f[0] += powz+powz*tmp*tmp ;
      //f[0] += pow(z[i],0.5)+pow(z[i],0.5)*tmp*tmp ;
    }
	f[0] = f[0]*f[0]/(nx-1)/(nx-1);
}

void ackley_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Ackley's  */
{
	shiftfunc(x, y, nx, Os);
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (int i=0; i<nx; i++)
		z[i]=y[i];

	asyfunc (z, y, nx, 0.5);
	for (int i=0; i<nx; i++)
		z[i] = y[i]*pow(10.0,1.0*i/(nx-1)/2.0);
	if (r_flag)
	rotatefunc(z, y, nx, &Mr[nx*nx]);
	else
    for (int i=0; i<nx; i++)
		y[i]=z[i];

    double sum1 = 0.0;
    double sum2 = 0.0;
    for (int i=0; i<nx; i++)
    {
        sum1 += y[i]*y[i];
        sum2 += cos(TWOPI*y[i]);
    }
    sum1 = -0.2*sqrt(sum1/nx);
    sum2 /= nx;
    f[0] =  E - 20.0*exp(sum1) - exp(sum2) + 20.0;
}

#define WEIERSTRASS_STRETCH 0.005
void weierstrass_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Weierstrass's  */
{
    int i,j;
    double sum,sum2;

	shiftfunc(x, y, nx, Os);
	for (i=0; i<nx; i++)//shrink to the orginal search range
    {
        y[i]*=WEIERSTRASS_STRETCH;
    }
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

	asyfunc (z, y, nx, 0.5);
	for (i=0; i<nx; i++)
		z[i] = y[i]*pow(10.0,1.0*i/(nx-1)/2.0);
	if (r_flag==1)
	rotatefunc(z, y, nx, &Mr[nx*nx]);
	else
    for (i=0; i<nx; i++)
		y[i]=z[i];

    const double a = 0.5;
    const double b = 3.0;
    int k_max = 20;
    f[0] = 0.0;
    for (i=0; i<nx; i++)
    {
        sum = 0.0;
		sum2 = 0.0;
        for (j=0; j<=k_max; j++)
        {
            sum += pow(a,j)*cos(TWOPI*pow(b,j)*(y[i]+0.5));
			sum2 += pow(a,j)*cos(TWOPI*pow(b,j)*0.5);
        }
        f[0] += sum;
    }
	f[0] -= nx*sum2;
}

#define GRIEWANK_STRETCH 6.0
void griewank_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Griewank's  */
{

	shiftfunc(x, y, nx, Os);
	for (int i=0; i<nx; i++)//shrink to the orginal search range
    {
        y[i]*=GRIEWANK_STRETCH;
    }
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (int i=0; i<nx; i++)
		z[i]=y[i];

	for (int i=0; i<nx; i++)
		z[i] = z[i]*pow(100.0,1.0*i/(nx-1)/2.0);


    double s = 0.0;
    double p = 1.0;
    for (int i=0; i<nx; i++)
    {
        s += z[i]*z[i];
        p *= cos(z[i]/sqrt(1.0+i));
    }
    f[0] = 1.0 + s/4000.0 - p;
}

#define RASTRIGIN_SHRINK 0.0512
void rastrigin_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Rastrigin's  */
{
    int i;
	const double alpha=10.0,beta=0.2;
	shiftfunc(x, y, nx, Os);
	for (i=0; i<nx; i++)//shrink to the orginal search range
    {
        y[i] *= RASTRIGIN_SHRINK;
        //y[i]=y[i]*RASTRIGIN_SHRINK;
    }

	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

    oszfunc (z, y, nx);
    asyfunc (y, z, nx, beta);

	if (r_flag)
	rotatefunc(z, y, nx, &Mr[nx*nx]);
	else
    for (i=0; i<nx; i++)
		y[i]=z[i];

	for (i=0; i<nx; i++)
	{
		y[i]*=pow(alpha,1.0*i/(nx-1)/2);
	}

	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

    f[0] = 0.0;
    for (i=0; i<nx; i++)
    {
        float zz = z[i];
        f[0] += (zz*zz - 10.0*cos(TWOPI*zz) + 10.0);
    }
}

void step_rastrigin_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Noncontinuous Rastrigin's  */
{
    int i;
	double alpha=10.0,beta=0.2;
	shiftfunc(x, y, nx, Os);
	for (i=0; i<nx; i++)//shrink to the orginal search range
    {
        y[i]=y[i]*5.12/100;
    }

	if (r_flag==1)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

    for (i=0; i<nx; i++)
	{
		if (fabs(z[i])>0.5)
		z[i]=floor(2*z[i]+0.5)/2;
	}

    oszfunc (z, y, nx);
    asyfunc (y, z, nx, beta);

	if (r_flag==1)
	rotatefunc(z, y, nx, &Mr[nx*nx]);
	else
    for (i=0; i<nx; i++)
		y[i]=z[i];

	for (i=0; i<nx; i++)
	{
		y[i]*=pow(alpha,1.0*i/(nx-1)/2);
	}

	if (r_flag==1)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

    f[0] = 0.0;
    for (i=0; i<nx; i++)
    {
        f[0] += (z[i]*z[i] - 10.0*cos(TWOPI*z[i]) + 10.0);
    }
}

#define SCHWEFEL_SHRINK 10.0
void schwefel_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Schwefel's  */
{
	double tmp;
	shiftfunc(x, y, nx, Os);
	for (int i=0; i<nx; i++)//shrink to the orginal search range
    {
        y[i]*=SCHWEFEL_SHRINK;
    }
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (int i=0; i<nx; i++)
		z[i]=y[i];

	for (int i=0; i<nx; i++)
		y[i] = z[i]*pow(10.0,1.0*i/(nx-1)/2.0);

	for (int i=0; i<nx; i++)
		z[i] = y[i]+4.209687462275036e+002;
	
    f[0]=0;
    for (int i=0; i<nx; i++)
	{
		if (z[i]>500)
		{
			f[0]-=(500.0-fmod(z[i],500))*sin(pow(500.0-fmod(z[i],500),0.5));
			tmp=(z[i]-500.0)/100;
			f[0]+= tmp*tmp/nx;
		}
		else if (z[i]<-500)
		{
			f[0]-=(-500.0+fmod(fabs(z[i]),500))*sin(pow(500.0-fmod(fabs(z[i]),500),0.5));
			tmp=(z[i]+500.0)/100;
			f[0]+= tmp*tmp/nx;
		}
		else
			f[0]-=z[i]*sin(pow(fabs(z[i]),0.5));
    }
    f[0]=4.189828872724338e+002*nx+f[0];
}

void katsuura_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Katsuura  */
{
    int i,j;
	double temp,tmp1,tmp2,tmp3;
	tmp3=pow(1.0*nx,1.2);
	shiftfunc(x, y, nx, Os);
	for (i=0; i<nx; i++)//shrink to the orginal search range
    {
        y[i]*=5.0/100.0;
    }
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

	for (i=0; i<nx; i++)
		z[i] *=pow(100.0,1.0*i/(nx-1)/2.0);

	if (r_flag)
	rotatefunc(z, y, nx, &Mr[nx*nx]);
	else
    for (i=0; i<nx; i++)
		y[i]=z[i];

    f[0]=1.0;
    for (i=0; i<nx; i++)
	{
		temp=0.0;
		for (j=1; j<=32; j++)
		{
			tmp1=pow(2.0,j);
			tmp2=tmp1*y[i];
			temp += fabs(tmp2-floor(tmp2+0.5))/tmp1;
		}
		f[0] *= pow(1.0+(i+1)*temp,10.0/tmp3);
    }
	tmp1=10.0/nx/nx;
    f[0]=f[0]*tmp1-tmp1;

}

#define BIRASTRIGIN_SHRINK 0.1
void bi_rastrigin_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Lunacek Bi_rastrigin Function */
{
    int i;
	double mu0=2.5,d=1.0,s,mu1,tmp;
	double *tmpx;
	tmpx=(double *)malloc(sizeof(double)  *  nx);
	s=1.0-1.0/(2.0*pow(nx+20.0,0.5)-8.2);
	mu1=-pow((mu0*mu0-d)/s,0.5);

	shiftfunc(x, y, nx, Os);
	for (i=0; i<nx; i++)//shrink to the orginal search range
    {
        y[i]*=BIRASTRIGIN_SHRINK;
    }

	for (i = 0; i < nx; i++)
    {
		tmpx[i]=2.0*y[i];
        if (Os[i] < 0.)
            tmpx[i] *= -1.;
    }

	for (i=0; i<nx; i++)
	{
		z[i]=tmpx[i];
		tmpx[i] += mu0;
	}
	if (r_flag)
	rotatefunc(z, y, nx, Mr);
	else
    for (i=0; i<nx; i++)
		y[i]=z[i];

	for (i=0; i<nx; i++)
		y[i] *=pow(100.0,1.0*i/(nx-1)/2.0);
	if (r_flag)
	rotatefunc(y, z, nx, &Mr[nx*nx]);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

	double tmp1=0,tmp2=0;
    for (i=0; i<nx; i++)
	{
		tmp = tmpx[i]-mu0;
		tmp1 += tmp*tmp;
		tmp = tmpx[i]-mu1;
		tmp2 += tmp*tmp;
    }
	tmp2 *= s;
	tmp2 += d*nx;
	tmp=0;
	for (i=0; i<nx; i++)
	{
		tmp+=cos(TWOPI*z[i]);
    }
	
	if(tmp1<tmp2)
		f[0] = tmp1;
	else
		f[0] = tmp2;
	f[0] += 10.0*(nx-tmp);
	free(tmpx);
}

#define GRIE_ROSEN_SHRINK 0.05
void grie_rosen_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Griewank-Rosenbrock  */
{
    int i;
    double temp,tmp1,tmp2;

	shiftfunc(x, y, nx, Os);
	for (i=0; i<nx; i++)//shrink to the orginal search range
    {
        y[i]=y[i]*GRIE_ROSEN_SHRINK;
    }
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

	for (i=0; i<nx; i++)//shift to orgin
    {
        z[i]=y[i]+1;
    }

    f[0]=0.0;
    for (i=0; i<nx-1; i++)
    {
		tmp1 = z[i]*z[i]-z[i+1];
		tmp2 = z[i]-1.0;
        temp = 100.0*tmp1*tmp1 + tmp2*tmp2;
         f[0] += (temp*temp)/4000.0 - cos(temp) + 1.0;
    }
	tmp1 = z[nx-1]*z[nx-1]-z[0];
	tmp2 = z[nx-1]-1.0;
    temp = 100.0*tmp1*tmp1 + tmp2*tmp2;;
    f[0] += (temp*temp)/4000.0 - cos(temp) + 1.0 ;
}


void escaffer6_func (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Expanded Scaffer¡¯s F6  */
{
    int i;
    double temp1, temp2;
	shiftfunc(x, y, nx, Os);
	if (r_flag)
	rotatefunc(y, z, nx, Mr);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

	asyfunc (z, y, nx, 0.5);
	if (r_flag)
	rotatefunc(y, z, nx, &Mr[nx*nx]);
	else
    for (i=0; i<nx; i++)
		z[i]=y[i];

    f[0] = 0.0;
    for (i=0; i<nx-1; i++)
    {
        temp1 = sin(sqrt(z[i]*z[i]+z[i+1]*z[i+1]));
		temp1 =temp1*temp1;
        temp2 = 1.0 + 0.001*(z[i]*z[i]+z[i+1]*z[i+1]);
        f[0] += 0.5 + (temp1-0.5)/(temp2*temp2);
    }
    temp1 = sin(sqrt(z[nx-1]*z[nx-1]+z[0]*z[0]));
	temp1 =temp1*temp1;
    temp2 = 1.0 + 0.001*(z[nx-1]*z[nx-1]+z[0]*z[0]);
    f[0] += 0.5 + (temp1-0.5)/(temp2*temp2);
}


void cf01 (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Composition Function 1 */
{
	int i,cf_num=5;
	double fit[5];
	double delta[5] = {10.0, 20.0, 30.0, 40.0, 50.0};
	double bias[5] = {0.0, 100.0, 200.0, 300.0, 400.0};
	
	i=0;
	rosenbrock_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	fit[i]=10000*fit[i]/1e+4;
	i=1;
	dif_powers_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	fit[i]=10000*fit[i]/1e+10;
	i=2;
	bent_cigar_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	fit[i]=10000*fit[i]/1e+30;
	i=3;
	discus_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	fit[i]=10000*fit[i]/1e+10;
	i=4;
	sphere_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],0);
	fit[i]=10000*fit[i]/1e+5;
	cf_cal(x, f, nx, Os, delta,bias,fit,cf_num);
}

#define CF_NUM_2 3
void cf02 (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Composition Function 2 */
{
	//const int cf_num=3;
	double fit[3];
	double delta[3] = {20.0,20.0,20.0};
	double bias[3] = {0.0, 100.0, 200.0};
	for(int i=0;i<CF_NUM_2;i++)
	{
		schwefel_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	}
	cf_cal(x, f, nx, Os, delta,bias,fit,CF_NUM_2);
}

#define CF_NUM_3 3
void cf03 (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Composition Function 3 */
{
	double fit[3];
	double delta[3] = {20.0,20.0,20.0};
	double bias[3] = {0.0, 100.0, 200.0};
	for(int i=0;i<CF_NUM_3;i++)
	{
		schwefel_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	}
	cf_cal(x, f, nx, Os, delta,bias,fit,CF_NUM_3);
}

#define CF_NUM_4 3
void cf04 (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Composition Function 4 */
{
	double fit[3];
	double delta[3] = {20.0,20.0,20.0};
	double bias[3] = {0.0, 100.0, 200.0};
	schwefel_func(x,&fit[0],nx,&Os[0],&Mr[0],r_flag);
	fit[0]=1000.0*fit[0]/4e+3;
	rastrigin_func(x,&fit[1],nx,&Os[nx],&Mr[nx*nx],r_flag);
	fit[1]=1000.0*fit[1]/1e+3;
	weierstrass_func(x,&fit[2],nx,&Os[2*nx],&Mr[2*nx*nx],r_flag);
	fit[2]=1000.0*fit[2]/400.0;
	cf_cal(x, f, nx, Os, delta,bias,fit,CF_NUM_4);
}

#define CF_NUM_5 3
void cf05 (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Composition Function 4 */
{
	double fit[3];
	double delta[3] = {10.0,30.0,50.0};
	double bias[3] = {0.0, 100.0, 200.0};
	schwefel_func(x,&fit[0],nx,&Os[0],&Mr[0],r_flag);
	rastrigin_func(x,&fit[1],nx,&Os[nx],&Mr[nx*nx],r_flag);
	weierstrass_func(x,&fit[2],nx,&Os[2*nx],&Mr[2*nx*nx],r_flag);
	fit[0]=1000.0*fit[0]/4e+3;
	fit[1]=1000.0*fit[1]/1e+3;
	fit[2]=1000.0*fit[2]/400.0;
	cf_cal(x, f, nx, Os, delta,bias,fit,CF_NUM_5);
}

#define CF_NUM_6 5
void cf06 (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Composition Function 6 */
{
	int i;
	double fit[5];
	double delta[5] = {10.0,10.0,10.0,10.0,10.0};
	double bias[5] = {0.0, 100.0, 200.0, 300.0, 400.0};
	i=0;
	schwefel_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	fit[i]=1000*fit[i]/4e+3;
	i=1;
	rastrigin_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	fit[i]=1000*fit[i]/1e+3;
	i=2;
	ellips_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	fit[i]=1000*fit[i]/1e+10;
	i=3;
	weierstrass_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	fit[i]=1000*fit[i]/400;
	i=4;
	griewank_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx*nx],r_flag);
	fit[i]=1000*fit[i]/100;
	cf_cal(x, f, nx, Os, delta,bias,fit,CF_NUM_6);

}

#define CF_NUM_7 5
void cf07 (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Composition Function 7 */
{
	int i, nx2=nx*nx;
	double fit[5];
	double delta[5] = {10.0,10.0,10.0,20.0,20.0};
	double bias[5] = {0.0, 100.0, 200.0, 300.0, 400.0};
	i=0;
	griewank_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],r_flag);
	fit[i]=10000.0*fit[i]/100;
	i=1;
	rastrigin_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],r_flag);
	fit[i]=10000.0*fit[i]/1e+3;
	i=2;
	schwefel_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],r_flag);
	fit[i]=10000.0*fit[i]/4e+3;
	i=3;
	weierstrass_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],r_flag);
	fit[i]=10000.0*fit[i]/400;
	i=4;
	sphere_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],0);
	fit[i]=10000.0*fit[i]/1e+5;
	cf_cal(x, f, nx, Os, delta,bias,fit,CF_NUM_7);
}

#define CF_NUM_8 5
void cf08 (double *x, double *f, int nx, double *Os,double *Mr,int r_flag) /* Composition Function 8 */
{
	int i, nx2=nx*nx;
	double fit[5];
	double delta[5] = {10.0,20.0,30.0,40.0,50.0};
	double bias[5] = {0.0, 100.0, 200.0, 300.0, 400.0};
	i=0;
	grie_rosen_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],r_flag);
	fit[i]=10000.0*fit[i]/4e+3;
	i=1;
	schaffer_F7_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],r_flag);
	fit[i]=10000.0*fit[i]/4e+6;
	i=2;
	schwefel_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],r_flag);
	fit[i]=10000.0*fit[i]/4e+3;
	i=3;
	escaffer6_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],r_flag);
	fit[i]=10000.0*fit[i]/2e+7;
	i=4;
	sphere_func(x,&fit[i],nx,&Os[i*nx],&Mr[i*nx2],0);
	fit[i]=10000.0*fit[i]/1e+5;
	cf_cal(x, f, nx, Os, delta,bias,fit,CF_NUM_8);
}

void shiftfunc (double *x, double *xshift, int nx,double *Os)
{
    for (int i=0; i<nx; i++)
    {
        xshift[i]=x[i]-Os[i];
    }
}

void rotatefunc (double *x, double *xrot, int nx,double *Mr)
{
    for (int i=0; i<nx; i++)
    {
        xrot[i]=0;
		for (int j=0; j<nx; j++)
		{
			xrot[i]=xrot[i]+x[j]*Mr[i*nx+j];
		}
    }
}

void asyfunc (double *x, double *xasy, int nx, double beta)
{
    for (int i=0; i<nx; i++)
    {
		if (x[i]>0)
        xasy[i]=pow(x[i],1.0+beta*i/(nx-1)*pow(x[i],0.5));
    }
}

void oszfunc (double *x, double *xosz, int nx)
{
	double c1,c2,xx;
    for (int i=0; i<nx; i++)
    {
		if (i==0||i==nx-1)
        {
			if (x[i]!=0)
				xx=log(fabs(x[i]));
			if (x[i]>0)
			{
				c1=10.0;
				c2=7.9;
			}
			else
			{
				c1=5.5;
				c2=3.1;
			}
			if (x[i]>0)
				xosz[i]=exp(xx+0.049*(sin(c1*xx)+sin(c2*xx))); //sx=1;
			else if (x[i]==0)
				xosz[i]=0; // sx=0;
			else
				xosz[i]=-exp(xx+0.049*(sin(c1*xx)+sin(c2*xx))); //sx=-1;
			//xosz[i]=sx*exp(xx+0.049*(sin(c1*xx)+sin(c2*xx)));
		}
		else
			xosz[i]=x[i];
    }
}


void cf_cal(double *x, double *f, int nx, double *Os,double * delta,double * bias,double * fit, int cf_num)
{
	double w_max=0.0,w_sum=0.0;
	for (int i=0; i<cf_num; i++)
	{
		fit[i] += bias[i];
		w_cf_func[i]=0;
		for (int j=0; j<nx; j++)
		{
			w_cf_func[i]+=pow(x[j]-Os[i*nx+j],2.0);
		}
		if (w_cf_func[i]!=0)
			w_cf_func[i]=pow(1.0/w_cf_func[i],0.5)*exp(-w_cf_func[i]/2.0/nx/pow(delta[i],2.0));
		else
			w_cf_func[i]=INF;
		if (w_cf_func[i]>w_max)
			w_max=w_cf_func[i];
	}

	for (int i=0; i<cf_num; i++)
	{
		w_sum=w_sum+w_cf_func[i];
	}
	if(w_max==0)
	{
		for (int i=0; i<cf_num; i++)
			w_cf_func[i]=1.0;
		w_sum=cf_num;
	}
	f[0] = 0.0;
    for (int i=0; i<cf_num; i++)
    {
		f[0]+=(w_cf_func[i]/w_sum*fit[i]);
    }
}


void test_func(struct DEPSOPosition *pop, int nx, int mx,int func_num)
{
	int cf_num=10,i;
	if (ini_flag==1)
	{
		// If I read from file everytime I change function, it will never end
		//if ((n_flag!=nx)||(func_flag!=func_num))
		if (n_flag!=nx)
			ini_flag=0;
		if (func_flag!=func_num)
			func_flag=func_num;
	}

	if (ini_flag==0)
	{
		FILE *fpt;
		char FileName[30];
		free(M);
		free(OShift);
		free(y);
		free(z);
		free(x_bound);
		free(w_cf_func);
		y=(double *)malloc(sizeof(double)  *  nx);
		z=(double *)malloc(sizeof(double)  *  nx);
		x_bound=(double *)malloc(sizeof(double)  *  nx);
		for (i=0; i<nx; i++)
			x_bound[i]=100.0;
			

		if (!(nx==2||nx==5||nx==10||nx==20||nx==30||nx==40||nx==50||nx==60||nx==70||nx==80||nx==90||nx==100))
		{
			printf("\nError: Test functions are only defined for D=2,5,10,20,30,40,50,60,70,80,90,100.\n");
		}
		
		sprintf(FileName, "input_data/M_D%d.txt", nx);
		fpt = fopen(FileName,"r");
		if (fpt==NULL)
		{
		    printf("\n Error: Cannot open input file for reading \n");
		}

		w_cf_func=(double *)malloc(cf_num * sizeof(double));

		M=(double*)malloc(cf_num*nx*nx*sizeof(double));
		if (M==NULL)
			printf("\nError: there is insufficient memory available!\n");
		for (i=0; i<cf_num*nx*nx; i++)
		{
            fscanf(fpt,"%lf",&M[i]);
		}
		fclose(fpt);
		

		fpt=fopen("input_data/shift_data.txt","r");
		if (fpt==NULL)
		{
			printf("\n Error: Cannot open input file for reading \n");
		}
		OShift=(double *)malloc(nx*cf_num*sizeof(double));
		if (OShift==NULL)
			printf("\nError: there is insufficient memory available!\n");
		for(i=0;i<cf_num*nx;i++)
		{
            fscanf(fpt,"%lf",&OShift[i]);
		}
		fclose(fpt);

		n_flag=nx;
		func_flag=func_num;
		ini_flag=1;
		//printf("Function has been initialized!\n");
	}


	for (i = 0; i < mx; i++)
	{
        DEPSOPosition *pos = &(pop[i]);
        double *x = pos->x;
        double *f = &(pos->fitness);
		switch(func_num)
		{
		case SPHERE_ID:	
			sphere_func(x, f,nx,OShift,M,0);
			//*f+=-1400.0;
			break;
		case ELLIPTIC_ID:	
			ellips_func(x, f,nx,OShift,M,0);
			//*f+=-1300.0;
			break;
		case ROTATED_ELLIPTIC_ID:	
			ellips_func(x, f,nx,OShift,M,1);
			//*f+=-1300.0;
			break;
		case ROTATED_CIGAR_ID:	
			bent_cigar_func(x, f,nx,OShift,M,1);
			//*f+=-1200.0;
			break;
		case ROTATED_DISCUS_ID:	
			discus_func(x, f,nx,OShift,M,1);
			//*f+=-1100.0;
			break;
		case POWERS_ID:
			dif_powers_func(x, f,nx,OShift,M,0);
			//*f+=-1000.0;
			break;
		case ROSENBROCK_ID:
			rosenbrock_func(x, f,nx,OShift,M,0);
			//*f+=-900.0;
			break;
		case ROTATED_ROSENBROCK_ID:
			rosenbrock_func(x, f,nx,OShift,M,1);
			//*f+=-900.0;
			break;
		case ROTATED_SCHAFFERS_F7_ID:	
			schaffer_F7_func(x, f,nx,OShift,M,1);
			//*f+=-800.0;
			break;
		case ACKLEY_ID:	
			ackley_func(x, f,nx,OShift,M,0);
			//*f+=-700.0;
			break;
		case ROTATED_ACKLEY_ID:	
			ackley_func(x, f,nx,OShift,M,1);
			//*f+=-700.0;
			break;
		case ROTATED_WEIERSTRASS_ID:	
			weierstrass_func(x, f,nx,OShift,M,1);
			//*f+=-600.0;
			break;
		case GRIEWANK_ID:	
			griewank_func(x, f,nx,OShift,M,0);
			//*f+=-500.0;
			break;
		case ROTATED_GRIEWANK_ID:	
			griewank_func(x, f,nx,OShift,M,1);
			//*f+=-500.0;
			break;
		case RASTRIGIN_ID:	
			rastrigin_func(x, f,nx,OShift,M,0);
			//*f+=-400.0;
			break;
		case ROTATED_RASTRIGIN_ID:	
			rastrigin_func(x, f,nx,OShift,M,1);
			//*f+=-300.0;
			break;
		case ROTATED_STEP_RASTRIGIN_ID:	
			step_rastrigin_func(x, f,nx,OShift,M,1);
			//*f+=-200.0;
			break;
		case SCHWEFEL_ID:	
			schwefel_func(x, f,nx,OShift,M,0);
			//*f+=-100.0;
			break;
		case ROTATED_SCHWEFEL_ID:	
			schwefel_func(x, f,nx,OShift,M,1);
			//*f+=100.0;
			break;
		case ROTATED_KATSUURA_ID:	
			katsuura_func(x, f,nx,OShift,M,1);
			//*f+=200.0;
			break;
		case LUNACEK_ID:	
			bi_rastrigin_func(x, f,nx,OShift,M,0);
			//*f+=300.0;
			break;
		case ROTATED_LUNACEK_ID:	
			bi_rastrigin_func(x, f,nx,OShift,M,1);
			//*f+=400.0;
			break;
		case GRIE_ROSEN_ID:	
			grie_rosen_func(x, f,nx,OShift,M,1);
			//*f+=500.0;
			break;
		case SCAFFER_ID:	
			escaffer6_func(x, f,nx,OShift,M,1);
			//*f+=600.0;
			break;
		case COMP1_ID:	
			cf01(x, f,nx,OShift,M,1);
			//*f+=700.0;
			break;
		case COMP2_ID:	
			cf02(x, f,nx,OShift,M,0);
			//*f+=800.0;
			break;
		case COMP3_ID:	
			cf03(x, f,nx,OShift,M,1);
			//*f+=900.0;
			break;
		case COMP4_ID:	
			cf04(x, f,nx,OShift,M,1);
			//*f+=1000.0;
			break;
		case COMP5_ID:	
			cf05(x, f,nx,OShift,M,1);
			//*f+=1100.0;
			break;
		case COMP6_ID:
			cf06(x, f,nx,OShift,M,1);
			//*f+=1200.0;
			break;
		case COMP7_ID:
			cf07(x, f,nx,OShift,M,1);
			//*f+=1300.0;
			break;
		case COMP8_ID:
			cf08(x, f,nx,OShift,M,1);
			//*f+=1400.0;
			break;
		default:
			printf("\nError: Function with ID #%d is not in this test suite!\n", func_num);
			exit(1);
		}
	}
}
