/*
  CEC13 Test Function Suite 
  Jane Jing Liang (email: liangjing@zzu.edu.cn) 
  Last Modified on 14th Feb. 2013
*/

#ifndef __TEST_FUNC_H__
#define __TEST_FUNC_H__

//#include <WINDOWS.H>      
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <iostream>
#include <string>

#include "common.h"

// CEC 2013 functions
#define SPHERE_ID                  1
#define ROTATED_ELLIPTIC_ID        2
#define ROTATED_CIGAR_ID           3
#define ROTATED_DISCUS_ID          4
#define POWERS_ID                  5
#define ROTATED_ROSENBROCK_ID      6  
#define ROTATED_SCHAFFERS_F7_ID    7
#define ROTATED_ACKLEY_ID          8
#define ROTATED_WEIERSTRASS_ID     9
#define ROTATED_GRIEWANK_ID       10
#define RASTRIGIN_ID              11
#define ROTATED_RASTRIGIN_ID      12
#define ROTATED_STEP_RASTRIGIN_ID 13
#define SCHWEFEL_ID               14
#define ROTATED_SCHWEFEL_ID       15
#define ROTATED_KATSUURA_ID       16
#define LUNACEK_ID                17
#define ROTATED_LUNACEK_ID        18
#define GRIE_ROSEN_ID             19
#define SCAFFER_ID                20
#define COMP1_ID                  21
#define COMP2_ID                  22
#define COMP3_ID                  23
#define COMP4_ID                  24
#define COMP5_ID                  25
#define COMP6_ID                  26
#define COMP7_ID                  27
#define COMP8_ID                  28
// not present in CEC 2013
#define ROSENBROCK_ID             29
#define GRIEWANK_ID               30
#define ACKLEY_ID                 31
#define ELLIPTIC_ID               32
// to be implemented (see fitness_functions.cpp)
#define ZAKHAROV_ID               -1  // -1 means not implemented
#define SCHWEFEL_1_2_ID           -1
#define SALOMON_ID                -1
#define SCHWEFEL_2_22_ID          -1
#define LEVY_ID                   -1
#define ALPINE_ID                 -1
#define DIXONPRICE_ID             -1
#define HYPERELLIPSOID_ID         -1

void sphere_func (double *, double *, int , double *,double *, int); /* Sphere */
void ellips_func(double *, double *, int , double *,double *, int); /* Ellipsoidal */
void bent_cigar_func(double *, double *, int , double *,double *, int); /* Discus */
void discus_func(double *, double *, int , double *,double *, int);  /* Bent_Cigar */
void dif_powers_func(double *, double *, int , double *,double *, int);  /* Different Powers */
void rosenbrock_func (double *, double *, int , double *,double *, int); /* Rosenbrock's */
void schaffer_F7_func (double *, double *, int , double *,double *, int); /* Schwefel's F7 */
void ackley_func (double *, double *, int , double *,double *, int); /* Ackley's */
void rastrigin_func (double *, double *, int , double *,double *, int); /* Rastrigin's  */
void weierstrass_func (double *, double *, int , double *,double *, int); /* Weierstrass's  */
void griewank_func (double *, double *, int , double *,double *, int); /* Griewank's  */
void schwefel_func (double *, double *, int , double *,double *, int); /* Schwefel's */
void katsuura_func (double *, double *, int , double *,double *, int); /* Katsuura */
void bi_rastrigin_func (double *, double *, int , double *,double *, int); /* Lunacek Bi_rastrigin */
void grie_rosen_func (double *, double *, int , double *,double *, int); /* Griewank-Rosenbrock  */
void escaffer6_func (double *, double *, int , double *,double *, int); /* Expanded Scaffers F6  */
void step_rastrigin_func (double *, double *, int , double *,double *, int); /* Noncontinuous Rastrigin's  */
void cf01 (double *, double *, int , double *,double *, int); /* Composition Function 1 */
void cf02 (double *, double *, int , double *,double *, int); /* Composition Function 2 */
void cf03 (double *, double *, int , double *,double *, int); /* Composition Function 3 */
void cf04 (double *, double *, int , double *,double *, int); /* Composition Function 4 */
void cf05 (double *, double *, int , double *,double *, int); /* Composition Function 5 */
void cf06 (double *, double *, int , double *,double *, int); /* Composition Function 6 */
void cf07 (double *, double *, int , double *,double *, int); /* Composition Function 7 */
void cf08 (double *, double *, int , double *,double *, int); /* Composition Function 8 */

void shiftfunc (double*,double*,int,double*);
void rotatefunc (double*,double*,int, double*);
void asyfunc (double *, double *x, int, double);
void oszfunc (double *, double *, int);
void cf_cal(double *, double *, int, double *,double *,double *,double *,int);

/// Test function
/// \param pop array of DEPSOPosition elements (individuals). Fitnesses will be written here
/// \param nx element size (number of dimensions)
/// \param mx number of elements in pop
/// \param func_num function id (see get_function_id_by_name)
void test_func(struct DEPSOPosition *pop, int nx, int mx, int func_num);

// Utilities
/// Fills lower and upper bounds according to the function id
void boundaries_by_function_id(int id, double & bottom, double & top);
/// Fills lower and upper bounds according to the function name
void boundaries_by_function_name(std::string name, double & bottom, double & top);
/// Returns the ID of the function given its name
int get_function_id_by_name(std::string name);

#endif // __TEST_FUNC_H__
