/// This file contains some definitions used by NSGA-II to instantiate
/// DE- and PSO-based LL-EA. Some definitions are not used here, but
/// are parsed by the Python scripts. In this way, they are consistent
/// with modifications done in the code (e.g. change of coding, range)

// valid algorithms
#define ALGO_DE "DE"
#define ALGO_PSO "PSO"

#define DE_META_MUTATION_COUNT 3
#define DE_META_MUTATION_0 'BEST'
#define DE_META_MUTATION_1 'TARGET-TO-BEST'
#define DE_META_MUTATION_2 'RANDOM'
#define DE_META_CROSSOVER_COUNT 2
#define DE_META_CROSSOVER_0 'BINOMIAL'
#define DE_META_CROSSOVER_1 'EXPONENTIAL'

// the parameters in x will be read with these indices:
//#define DE_META_F_INDEX                 0
//#define DE_META_CR_INDEX                (DE_META_F_INDEX + 1)
//#define DE_META_POPULATION_INDEX        (DE_META_CR_INDEX + 1)
//#define DE_META_MUTATION_INDEX (DE_META_POPULATION_INDEX + 1)
//#define DE_META_CROSSOVER_INDEX         (DE_META_MUTATION_INDEX + DE_META_MUTATION_COUNT)
//#define DE_META_PARTICLE_SIZE           (DE_META_CROSSOVER_INDEX + DE_META_CROSSOVER_COUNT)
#define DE_META_F_INDEX         0
#define DE_META_CR_INDEX        1
#define DE_META_POPSIZE_INDEX   2
#define DE_META_MUTATION_INDEX  3
#define DE_META_CROSSOVER_INDEX 6
#define DE_META_PARTICLE_SIZE   8

// range of DE numerical parameters
#define DE_META_POPSIZE_MIN   4.0
#define DE_META_POPSIZE_MAX 300.0
#define DE_META_CR_MIN        0.0
#define DE_META_CR_MAX        1.0
#define DE_META_F_MIN         0.0
#define DE_META_F_MAX         2.0

// the five parameters in x will be read with these indices:
#define PSO_META_INERTIA_INDEX  0
#define PSO_META_CSELF_INDEX    1
#define PSO_META_CGLOBAL_INDEX  2
#define PSO_META_POPSIZE_INDEX  3
#define PSO_META_TOPOLOGY_INDEX 4
#define PSO_META_PARTICLE_SIZE  7

#define PSO_META_TOPOLOGY_COUNT 3
#define PSO_META_TOPOLOGY_0 'RING'
#define PSO_META_TOPOLOGY_1 'STAR'
#define PSO_META_TOPOLOGY_2 'GLOBAL'

// range of DE numerical parameters
#define PSO_META_POPSIZE_MIN   4.0
#define PSO_META_POPSIZE_MAX 300.0
#define PSO_META_INERTIA_MIN  -0.5
#define PSO_META_INERTIA_MAX   1.5
#define PSO_META_CSELF_MIN    -0.5
#define PSO_META_CSELF_MAX     4.0
#define PSO_META_CGLOBAL_MIN  -0.5
#define PSO_META_CGLOBAL_MAX   4.0
