#ifndef __PARAMETER_PARSER_CPP__
#define __PARAMETER_PARSER_CPP__


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "parametersParser.h"


/*

********************************
Example of parameters parsing...
********************************

int parseAllParameters(char* fileName){

	double value = 0;
	char tmp[100];
	if(simpleParse(fileName, "BASE_DIR", tmp))
		return -1;
	string baseDir = tmp;

	double doubleValue;
	if(simpleParse(fileName, "COEFF", &doubleValue))
		doubleValue = value;
	else
		return -1;

	int intValue;
	if(simpleParse(fileName, "IMG_WIDTH", &intValue))
		intValue = value;
	else
		return -1;

	return 0;
}
*/


bool parametersParser::parse(const char* fileName, const char* parameterName, double* par){

	FILE *fp;
	if ((fp = fopen(fileName, "r")) == NULL){
		printf("\nError opening file %s!\n", fileName);
		return false;
	}

	char buffer[512];
	char* tok = 0;
	char* value = 0;
	while(fgets(buffer,512,fp)){
		tok = strchr(buffer,'#');
		if(tok){
			tok++;
			value = strchr(tok,' ');
			if (value){
				*value++ = 0;
				if(!strcmp(tok, parameterName)){
					if(!sscanf(value, "%lf", par)){
						fclose(fp);
						return false;
					}
					fclose(fp);
					return true;
				}
			}
		}
	}
	fclose(fp);
	//printf("Parameter %s not found!!!\n", parameterName);
	//exit(0);
	return false;
}


bool parametersParser::parse(const char* fileName, const char* parameterName, int* par){

	FILE *fp;
	if ((fp = fopen(fileName, "r")) == NULL){
		printf("\nError opening file %s!\n", fileName);
		return false;
	}

	char buffer[512];
	char* tok = 0;
	char* value = 0;
	while(fgets(buffer,512,fp)){
		tok = strchr(buffer,'#');
		if(tok){
			tok++;
			value = strchr(tok,' ');
			if (value){
				*value++ = 0;
				if(!strcmp(tok, parameterName)){
					if(!sscanf(value, "%d", par)){
						fclose(fp);
						return false;
					}
					fclose(fp);
					return true;
				}
			}
		}
	}
	fclose(fp);
	//printf("Parameter %s not found!!!\n", parameterName);
	//exit(0);
	return false;
}

bool parametersParser::parse(const char* fileName, const char* parameterName, char* par){

	FILE *fp;
	if ((fp = fopen(fileName, "r")) == NULL){
		printf("\nError opening file %s!\n", fileName);
		return false;
	}

	char buffer[512];
	char* tok = 0;
	char* value = 0;
	while(fgets(buffer,512,fp)){
		tok = strchr(buffer,'#');
		if(tok){
			tok++;
			value = strchr(tok,' ');
			if (value){
				*value++ = 0;
				if(!strcmp(tok, parameterName)){
					if(!sscanf(value, "%s", par)){
						fclose(fp);
						return false;
					}
					fclose(fp);
					return true;
				}
			}
		}
	}
	fclose(fp);
	//printf("Parameter %s not found!!!\n", parameterName);
	//exit(0);
	return false;
}

bool parametersParser::set_parameter(const char* fileName, const char* parameterName, int* par, bool mustExist, bool verbose)
{
	int tmp;
	bool ret;
	if(ret = parse(fileName, parameterName, &tmp))
	{
		*par = tmp;
		if(verbose)
			printf("%s set to %d\n", parameterName, *par);
	}
	else
    {
		if(verbose)
			printf("%s not found\n", parameterName);
        if(mustExist)
        {
            printf("Parameter %s must be set! Exiting!\n", parameterName);
            exit(1);
        }
    }
	return ret;
}

bool parametersParser::set_parameter(const char* fileName, const char* parameterName, double* par, bool mustExist, bool verbose)
{
	double tmp;
	bool ret;
	if(ret = parse(fileName, parameterName, &tmp))
	{
		*par = tmp;
		if(verbose)
			printf("%s set to %f\n", parameterName, *par);
	}
	else 
    {
		if(verbose)
			printf("%s not found\n", parameterName);
        if(mustExist)
        {
            printf("Parameter %s must be set! Exiting!\n", parameterName);
            exit(1);
        }
    }
	return ret;
}

bool parametersParser::set_parameter(const char* fileName, const char* parameterName, std::string* par, bool mustExist, bool verbose)
{
	char tmp[512];
	bool ret;
	if(ret = parse(fileName, parameterName, tmp))
	{
		*par = tmp;
		if(verbose)
			printf("%s set to %s\n", parameterName, par->c_str());
	}
	else 
    {
		if(verbose)
			printf("%s not found\n", parameterName);
        if(mustExist)
        {
            printf("Parameter %s must be set! Exiting!\n", parameterName);
            exit(1);
        }
    }
	return ret;
}
#endif
