
#ifndef _PARAMETERS_PARSER_H
#define _PARAMETERS_PARSER_H

#include <string>

//! \brief Utility class used to read a configuration file

//! Format of a configuration file \n

//! A Parameter starts with a '#' at the very beginning of the line
//! the name of the parameter must follows the '#' without any space! \n
//! \code
//! // this is a comment
//! #BASE_DIR /home/user/   //this is another comment...
//! #COEFF 3.2
//! #IMG_WIDTH 720
//! \endcode
class parametersParser{
  public:
	
    /// Reads a parameter from a configuration file
    /// \param fileName the configuration file
    /// \param parameterName the name of the parameter to read
    /// \param par the variable in which the value will be output
    /// \param mustExist bool, if true and the parameter is not in the file, the program will exit.
    ///  If false, par will be left unchanged
    /// \param verbose if true some information regarding the process will be printed
    /// \return true if the parameter have been found, false if not
	bool set_parameter(const char* fileName, const char* parameterName, int*    par, bool mustExist=false, bool verbose=true);
	bool set_parameter(const char* fileName, const char* parameterName, double* par, bool mustExist=false, bool verbose=true);
	bool set_parameter(const char* fileName, const char* parameterName, std::string* par, bool mustExist=false, bool verbose=true);
    
  private:
    bool parse(const char* fileName, const char* parameterName, double* par);
	bool parse(const char* fileName, const char* parameterName, int*    par);
	bool parse(const char* fileName, const char* parameterName, char*   par);
	
};





#endif


